﻿using System;
using System.Collections.Generic;
using System.Text;

using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections.Specialized;
using System.Configuration;

namespace YYControls
{
    /// <summary>
    /// SmartGridView类的属性部分
    /// </summary>
    public partial class SmartGridView:IPostBackDataHandler
    {
        private FixRowColumn _fixRowColumn;
        /// <summary>
        /// 固定指定行、指定列
        /// </summary>
        [
        Description("固定指定行、指定列"),
        Category("扩展"),
        DesignerSerializationVisibility(DesignerSerializationVisibility.Content),
        PersistenceMode(PersistenceMode.InnerProperty)
        ]
        public virtual FixRowColumn FixRowColumn
        {
            get
            {
                if (this._fixRowColumn == null)
                {
                    this._fixRowColumn = new FixRowColumn();
                }
                return this._fixRowColumn;
            }
            set { this._fixRowColumn = value; }
        }

        //增加两个私有变量记录滚动条的位置
        //滚动条X的位置
        private int _yy_SmartGridView_x;

        //滚动条Y的位置
        private int _yy_SmartGridView_y;

        //重写GridView的OnPreRender方法，用于注册两个HiddenField，
        //以及注册设置GridView的滚动条的位置的javascript代码
        protected override void OnPreRender(EventArgs e)
        {
            if (FixRowColumn.EnableScrollState)
            {
                //滚动条X的位置
                Page.ClientScript.RegisterHiddenField("yy_SmartGridView_x",_yy_SmartGridView_x.ToString());
                //滚动条Y的位置
                Page.ClientScript.RegisterHiddenField("yy_SmartGridView_y", _yy_SmartGridView_y.ToString());

                //设置GridView的滚动条位置
                Page.ClientScript.RegisterStartupScript(this.GetType(), "jsSetScroll", "<script type=\"text/javascript\">document.getElementById('yy_ScrollDiv').scrollLeft=" + _yy_SmartGridView_x + ";document.getElementById('yy_ScrollDiv').scrollTop=" + _yy_SmartGridView_y + ";</script>");
            }

            //将控件注册为要求在页回发至服务器时进行回发处理的控件
            if (Page != null)
            {
                Page.RegisterRequiresPostBack(this);
            }

            base.OnPreRender(e);
        }

        //重写GridView的Render方法，将GridView用一个div包裹起来
        protected override void RenderContents(HtmlTextWriter writer)
        {
            if (FixRowColumn.TableWidth != null || FixRowColumn.TableHeight != null)
            {
                if (FixRowColumn.TableWidth == null)
                {
                    FixRowColumn.TableWidth = (new Unit(100, UnitType.Percentage)).ToString();
                }
                if (FixRowColumn.TableHeight == null)
                {
                    FixRowColumn.TableHeight = (new Unit(100,UnitType.Percentage)).ToString();
                }

                writer.Write("<div id='yy_ScrollDiv' style=\"overflow: auto; width: "
                + FixRowColumn.TableWidth.ToString() + "; height: "
                + FixRowColumn.TableHeight.ToString() + "; position: relative;\" ");
                
                //如果保持滚动条位置的话，用隐藏字段记录滚动条的位置
                if (FixRowColumn.EnableScrollState)
                {
                    writer.Write("onscroll=\"document.getElementById('yy_SmartGridView_x').value = this.scrollLeft; document.getElementById('yy_SmartGridView_y').value = this.scrollTop;\">");
                }
                else
                {
                    writer.Write(">");
                }
            }

            base.RenderContents(writer);

            //Div结束
            if (FixRowColumn.TableWidth != null || FixRowColumn.TableHeight != null)
            {
                writer.Write("</div>");
            }
        }

        //获取存储了滚条位置信息的HiddenField的值
        void IPostBackDataHandler.RaisePostDataChangedEvent()
        { }

        bool IPostBackDataHandler.LoadPostData(string postDataKey, NameValueCollection postCollection)
        {
            //获取滚动条的位置信息
            _yy_SmartGridView_x = string.IsNullOrEmpty(postCollection["yy_SmartGridView_x"]) ? 0 : Convert.ToInt32(postCollection["yy_SmartGridView_x"]);
            _yy_SmartGridView_y = string.IsNullOrEmpty(postCollection["yy_SmartGridView_y"]) ? 0 : Convert.ToInt32(postCollection["yy_SmartGridView_y"]);
            return false;
        }

    }
}
